@@ -38,17 +38,22 @@
 #include <glib/gi18n.h>
 
 #include "gedit-app.h"
-#include "gedit-prefs-manager-app.h"
 #include "gedit-commands.h"
 #include "gedit-notebook.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
 #include "gedit-enum-types.h"
 #include "gedit-dirs.h"
-#include "gseal-gtk-compat.h"
+#include "gedit-settings.h"
 
 #ifdef OS_OSX
-#include <ige-mac-integration.h>
+#include "gedit-app-osx.h"
+#else
+#ifdef OS_WIN32
+#include "gedit-app-win32.h"
+#else
+#include "gedit-app-x11.h"
+#endif
 #endif
 
 #define GEDIT_PAGE_SETUP_FILE		"gedit-page-setup"
@@ -72,9 +77,12 @@ struct _GeditAppPrivate
 
 	GtkPageSetup      *page_setup;
 	GtkPrintSettings  *print_settings;
+	
+	GSettings         *settings;
+	GSettings         *window_settings;
 };
 
-G_DEFINE_TYPE(GeditApp, gedit_app, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE(GeditApp, gedit_app, G_TYPE_INITIALLY_UNOWNED)
 
 static void
 gedit_app_finalize (GObject *object)
@@ -91,6 +99,26 @@ gedit_app_finalize (GObject *object)
 	G_OBJECT_CLASS (gedit_app_parent_class)->finalize (object);
 }
 
+static void
+gedit_app_dispose (GObject *object)
+{
+	GeditApp *app = GEDIT_APP (object); 
+
+	if (app->priv->window_settings != NULL)
+	{
+		g_object_unref (app->priv->window_settings);
+		app->priv->window_settings = NULL;
+	}
+
+	if (app->priv->settings != NULL)
+	{
+		g_object_unref (app->priv->settings);
+		app->priv->settings = NULL;
+	}
+
+	G_OBJECT_CLASS (gedit_app_parent_class)->dispose (object);
+}
+
 static void
 gedit_app_get_property (GObject    *object,
 			guint       prop_id,
@@ -110,14 +138,128 @@ gedit_app_get_property (GObject    *object,
 	}
 }
 
-static void 
+static GObject *
+gedit_app_constructor (GType                  gtype,
+                       guint                  n_construct_params,
+                       GObjectConstructParam *construct_params)
+{
+	static GObject *app = NULL;
+
+	if (!app)
+	{
+		app = G_OBJECT_CLASS (gedit_app_parent_class)->constructor (gtype,
+		                                                            n_construct_params,
+		                                                            construct_params);
+
+		g_object_add_weak_pointer (app, (gpointer *) &app);
+
+		return app;
+	}
+
+	return g_object_ref (app);
+}
+
+static gboolean
+gedit_app_last_window_destroyed_impl (GeditApp *app)
+{
+	return TRUE;
+}
+
+static gchar *
+gedit_app_help_link_id_impl (GeditApp    *app,
+                             const gchar *name,
+                             const gchar *link_id)
+{
+	if (link_id)
+	{
+		return g_strdup_printf ("ghelp:%s?%s", name, link_id);
+	}
+	else
+	{
+		return g_strdup_printf ("ghelp:%s", name);
+	}
+}
+
+static gboolean
+gedit_app_show_help_impl (GeditApp    *app,
+                          GtkWindow   *parent,
+                          const gchar *name,
+                          const gchar *link_id)
+{
+	GError *error = NULL;
+	gboolean ret;
+	gchar *link;
+
+	if (name == NULL)
+	{
+		name = "gedit";
+	}
+	else if (strcmp (name, "gedit.xml") == 0)
+	{
+		g_warning ("%s: Using \"gedit.xml\" for the help name is deprecated, use \"gedit\" or simply NULL instead", G_STRFUNC);
+		name = "gedit";
+	}
+
+	link = GEDIT_APP_GET_CLASS (app)->help_link_id (app, name, link_id);
+
+	ret = gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (parent)),
+	                    link,
+	                    GDK_CURRENT_TIME,
+	                    &error);
+
+	g_free (link);
+
+	if (error != NULL)
+	{
+		GtkWidget *dialog;
+
+		dialog = gtk_message_dialog_new (parent,
+						 GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE, 
+						 _("There was an error displaying the help."));
+
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  "%s", error->message);
+
+		g_signal_connect (G_OBJECT (dialog),
+				  "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+		gtk_widget_show (dialog);
+
+		g_error_free (error);
+	}
+
+	return ret;
+}
+
+static void
+gedit_app_set_window_title_impl (GeditApp    *app,
+                                 GeditWindow *window,
+                                 const gchar *title)
+{
+	gtk_window_set_title (GTK_WINDOW (window), title);
+}
+
+static void
 gedit_app_class_init (GeditAppClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = gedit_app_finalize;
+	object_class->dispose = gedit_app_dispose;
 	object_class->get_property = gedit_app_get_property;
-	
+	object_class->constructor = gedit_app_constructor;
+
+	klass->last_window_destroyed = gedit_app_last_window_destroyed_impl;
+	klass->show_help = gedit_app_show_help_impl;
+	klass->help_link_id = gedit_app_help_link_id_impl;
+	klass->set_window_title = gedit_app_set_window_title_impl;
+
 	g_object_class_install_property (object_class,
 					 PROP_LOCKDOWN,
 					 g_param_spec_flags ("lockdown",
@@ -128,13 +270,13 @@ gedit_app_class_init (GeditAppClass *klass)
 							     G_PARAM_READABLE |
 							     G_PARAM_STATIC_STRINGS));
 
-	g_type_class_add_private (object_class, sizeof(GeditAppPrivate));
+	g_type_class_add_private (object_class, sizeof (GeditAppPrivate));
 }
 
 static gboolean
 ensure_user_config_dir (void)
 {
-	gchar *config_dir;
+	const gchar *config_dir;
 	gboolean ret = TRUE;
 	gint res;
 
@@ -152,8 +294,6 @@ ensure_user_config_dir (void)
 		ret = FALSE;
 	}
 
-	g_free (config_dir);
-
 	return ret;
 }
 
@@ -162,10 +302,12 @@ load_accels (void)
 {
 	gchar *filename;
 
-	filename = gedit_dirs_get_user_accels_file ();
+	filename = g_build_filename (gedit_dirs_get_user_config_dir (),
+				     "accels",
+				     NULL);
 	if (filename != NULL)
 	{
-		gedit_debug_message (DEBUG_APP, "Loading keybindings from %s\n", filename);		
+		gedit_debug_message (DEBUG_APP, "Loading keybindings from %s\n", filename);
 		gtk_accel_map_load (filename);
 		g_free (filename);
 	}
@@ -176,10 +318,12 @@ save_accels (void)
 {
 	gchar *filename;
 
-	filename = gedit_dirs_get_user_accels_file ();
+	filename = g_build_filename (gedit_dirs_get_user_config_dir (),
+				     "accels",
+				     NULL);
 	if (filename != NULL)
 	{
-		gedit_debug_message (DEBUG_APP, "Saving keybindings in %s\n", filename);		
+		gedit_debug_message (DEBUG_APP, "Saving keybindings in %s\n", filename);
 		gtk_accel_map_save (filename);
 		g_free (filename);
 	}
@@ -188,7 +332,7 @@ save_accels (void)
 static gchar *
 get_page_setup_file (void)
 {
-	gchar *config_dir;
+	const gchar *config_dir;
 	gchar *setup = NULL;
 
 	config_dir = gedit_dirs_get_user_config_dir ();
@@ -198,7 +342,6 @@ get_page_setup_file (void)
 		setup = g_build_filename (config_dir,
 					  GEDIT_PAGE_SETUP_FILE,
 					  NULL);
-		g_free (config_dir);
 	}
 
 	return setup;
@@ -261,7 +404,7 @@ save_page_setup (GeditApp *app)
 static gchar *
 get_print_settings_file (void)
 {
-	gchar *config_dir;
+	const gchar *config_dir;
 	gchar *settings = NULL;
 
 	config_dir = gedit_dirs_get_user_config_dir ();
@@ -271,7 +414,6 @@ get_print_settings_file (void)
 		settings = g_build_filename (config_dir,
 					     GEDIT_PRINT_SETTINGS_FILE,
 					     NULL);
-		g_free (config_dir);
 	}
 
 	return settings;
@@ -337,16 +479,13 @@ gedit_app_init (GeditApp *app)
 	app->priv = GEDIT_APP_GET_PRIVATE (app);
 
 	load_accels ();
+	
+	/* Load settings */
+	app->priv->settings = gedit_settings_new ();
+	app->priv->window_settings = g_settings_new ("org.gnome.gedit.state.window");
 
 	/* initial lockdown state */
-	app->priv->lockdown = gedit_prefs_manager_get_lockdown ();
-}
-
-static void
-app_weak_notify (gpointer data,
-		 GObject *where_the_app_was)
-{
-	gtk_main_quit ();
+	app->priv->lockdown = gedit_settings_get_lockdown (GEDIT_SETTINGS (app->priv->settings));
 }
 
 /**
@@ -360,18 +499,29 @@ app_weak_notify (gpointer data,
 GeditApp *
 gedit_app_get_default (void)
 {
-	static GeditApp *app = NULL;
+	GeditApp *app;
+	GType type;
 
-	if (app != NULL)
-		return app;
+#ifdef OS_OSX
+	type = GEDIT_TYPE_APP_OSX;
+#else
+#ifdef OS_WIN32
+	type = GEDIT_TYPE_APP_WIN32;
+#else
+	type = GEDIT_TYPE_APP_X11;
+#endif
+#endif
 
-	app = GEDIT_APP (g_object_new (GEDIT_TYPE_APP, NULL));	
+	app = GEDIT_APP (g_object_new (type, NULL));
 
-	g_object_add_weak_pointer (G_OBJECT (app),
-				   (gpointer) &app);
-	g_object_weak_ref (G_OBJECT (app),
-			   app_weak_notify,
-			   NULL);
+	if (g_object_is_floating (app))
+	{
+		g_object_ref_sink (app);
+	}
+	else
+	{
+		g_object_unref (app);
+	}
 
 	return app;
 }
@@ -409,10 +559,12 @@ window_delete_event (GeditWindow *window,
 	    (GEDIT_WINDOW_STATE_SAVING |
 	     GEDIT_WINDOW_STATE_PRINTING |
 	     GEDIT_WINDOW_STATE_SAVING_SESSION))
-	    	return TRUE;
-	
+	{
+		return TRUE;
+	}
+
 	_gedit_cmd_file_quit (NULL, window);
-		
+
 	/* Do not destroy the window */
 	return TRUE;
 }
@@ -441,14 +593,11 @@ window_destroy (GeditWindow *window,
 */
 	if (app->priv->windows == NULL)
 	{
-#ifdef OS_OSX
-		if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "gedit-is-quitting-all")))
+		if (!GEDIT_APP_GET_CLASS (app)->last_window_destroyed (app))
 		{
-			/* Create hidden proxy window on OS X to handle the menu */
-			gedit_app_create_window (app, NULL);
 			return;
 		}
-#endif
+
 		/* Last window is gone... save some settings and exit */
 		ensure_user_config_dir ();
 
@@ -456,7 +605,7 @@ window_destroy (GeditWindow *window,
 		save_page_setup (app);
 		save_print_settings (app);
 
-		g_object_unref (app);
+		gtk_main_quit ();
 	}
 }
 
@@ -524,20 +673,18 @@ gedit_app_create_window_real (GeditApp    *app,
 		GdkWindowState state;
 		gint w, h;
 
-		state = gedit_prefs_manager_get_window_state ();
+		state = g_settings_get_int (app->priv->window_settings,
+					    GEDIT_SETTINGS_WINDOW_STATE);
+
+		g_settings_get (app->priv->window_settings,
+				GEDIT_SETTINGS_WINDOW_SIZE,
+				"(ii)", &w, &h);
+		gtk_window_set_default_size (GTK_WINDOW (window), w, h);
 
 		if ((state & GDK_WINDOW_STATE_MAXIMIZED) != 0)
-		{
-			gedit_prefs_manager_get_default_window_size (&w, &h);
-			gtk_window_set_default_size (GTK_WINDOW (window), w, h);
 			gtk_window_maximize (GTK_WINDOW (window));
-		}
 		else
-		{
-			gedit_prefs_manager_get_window_size (&w, &h);
-			gtk_window_set_default_size (GTK_WINDOW (window), w, h);
 			gtk_window_unmaximize (GTK_WINDOW (window));
-		}
 
 		if ((state & GDK_WINDOW_STATE_STICKY ) != 0)
 			gtk_window_stick (GTK_WINDOW (window));
@@ -564,6 +711,7 @@ gedit_app_create_window_real (GeditApp    *app,
 /**
  * gedit_app_create_window:
  * @app: the #GeditApp
+ * @screen: (allow-none):
  *
  * Create a new #GeditWindow part of @app.
  *
@@ -604,8 +752,8 @@ _gedit_app_restore_window (GeditApp    *app,
  *
  * Returns all the windows currently present in #GeditApp.
  *
- * Return value: the list of #GeditWindows objects. The list
- * should not be freed
+ * Return value: (element-type Window) (transfer none): the list of #GeditWindows objects.
+ * The list should not be freed
  */
 const GList *
 gedit_app_get_windows (GeditApp *app)
@@ -747,7 +895,8 @@ _gedit_app_get_window_in_viewport (GeditApp  *app,
  *
  * Returns all the documents currently open in #GeditApp.
  *
- * Return value: a newly allocated list of #GeditDocument objects
+ * Return value: (element-type Gedit.Document) (transfer container):
+ * a newly allocated list of #GeditDocument objects
  */
 GList *
 gedit_app_get_documents	(GeditApp *app)
@@ -776,7 +925,8 @@ gedit_app_get_documents	(GeditApp *app)
  *
  * Returns all the views currently present in #GeditApp.
  *
- * Return value: a newly allocated list of #GeditView objects
+ * Return value: (element-type Gedit.View) (transfer container):
+ * a newly allocated list of #GeditView objects
  */
 GList *
 gedit_app_get_views (GeditApp *app)
@@ -815,6 +965,26 @@ gedit_app_get_lockdown (GeditApp *app)
 	return app->priv->lockdown;
 }
 
+gboolean
+gedit_app_show_help (GeditApp    *app,
+                     GtkWindow   *parent,
+                     const gchar *name,
+                     const gchar *link_id)
+{
+	g_return_val_if_fail (GEDIT_IS_APP (app), FALSE);
+	g_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), FALSE);
+
+	return GEDIT_APP_GET_CLASS (app)->show_help (app, parent, name, link_id);
+}
+
+void
+gedit_app_set_window_title (GeditApp    *app,
+                            GeditWindow *window,
+                            const gchar *title)
+{
+	GEDIT_APP_GET_CLASS (app)->set_window_title (app, window, title);
+}
+
 static void
 app_lockdown_changed (GeditApp *app)
 {
@@ -903,4 +1073,12 @@ _gedit_app_set_default_print_settings (GeditApp         *app,
 	app->priv->print_settings = g_object_ref (settings);
 }
 
+GSettings *
+_gedit_app_get_settings (GeditApp *app)
+{
+	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
+
+	return app->priv->settings;
+}
+
 /* ex:ts=8:noet: */
