@@ -48,10 +48,8 @@
 #include "gedit-tab-label.h"
 #include "gedit-marshal.h"
 #include "gedit-window.h"
-
-#ifdef BUILD_SPINNER
-#include "gedit-spinner.h"
-#endif
+#include "gedit-enum-types.h"
+#include "gedit-settings.h"
 
 #define AFTER_ALL_TABS -1
 #define NOT_IN_APP_WINDOWS -2
@@ -60,22 +58,29 @@
 
 struct _GeditNotebookPrivate
 {
+	GSettings     *ui_settings;
+
 	GList         *focused_pages;
 	gulong         motion_notify_handler_id;
+
 	gint           x_start;
 	gint           y_start;
-	gint           drag_in_progress : 1;
-	gint	       always_show_tabs : 1;
-	gint           close_buttons_sensitive : 1;
-	gint           tab_drag_and_drop_enabled : 1;
+
+	GeditNotebookShowTabsModeType show_tabs_mode;
+
+	gint           xthickness;
+
+	guint          drag_in_progress : 1;
+	guint          close_buttons_sensitive : 1;
+	guint          tab_drag_and_drop_enabled : 1;
+	guint          destroy_has_run : 1;
+	guint          collapse_border : 1;
 };
 
 G_DEFINE_TYPE(GeditNotebook, gedit_notebook, GTK_TYPE_NOTEBOOK)
 
-static void gedit_notebook_finalize (GObject *object);
-
-static gboolean gedit_notebook_change_current_page (GtkNotebook *notebook,
-						    gint         offset);
+static gboolean gedit_notebook_change_current_page (GtkNotebook   *notebook,
+						    gint           offset);
 
 static void move_current_tab_to_another_notebook  (GeditNotebook  *src,
 						   GeditNotebook  *dest,
@@ -85,11 +90,16 @@ static void move_current_tab_to_another_notebook  (GeditNotebook  *src,
 /* Local variables */
 static GdkCursor *cursor = NULL;
 
+/* Properties */
+enum
+{
+	PROP_0,
+	PROP_SHOW_TABS_MODE
+};
+
 /* Signals */
 enum
 {
-	TAB_ADDED,
-	TAB_REMOVED,
 	TABS_REORDERED,
 	TAB_DETACHED,
 	TAB_CLOSE_REQUEST,
@@ -98,36 +108,155 @@ enum
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+
+static void
+update_tabs_visibility (GeditNotebook *notebook,
+			gboolean       before_inserting)
+{
+	gboolean show_tabs;
+
+	switch (notebook->priv->show_tabs_mode)
+	{
+		case GEDIT_NOTEBOOK_SHOW_TABS_NEVER:
+			show_tabs = FALSE;
+			break;
+		case GEDIT_NOTEBOOK_SHOW_TABS_AUTO:
+			{
+				guint num;
+
+				num = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+
+				if (before_inserting)
+					++num;
+
+				show_tabs = num > 1;
+			}
+			break;
+		case GEDIT_NOTEBOOK_SHOW_TABS_ALWAYS:
+		default:
+			show_tabs = TRUE;
+			break;
+	}
+
+	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), show_tabs);
+}
+
+static void
+gedit_notebook_get_property (GObject    *object,
+			     guint       prop_id,
+			     GValue     *value,
+			     GParamSpec *pspec)
+{
+	GeditNotebook *notebook = GEDIT_NOTEBOOK (object);
+
+	switch (prop_id)
+	{
+		case PROP_SHOW_TABS_MODE:
+			g_value_set_enum (value,
+					  notebook->priv->show_tabs_mode);
+			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
+	}
+}
+
+static void
+gedit_notebook_set_property (GObject      *object,
+			     guint         prop_id,
+			     const GValue *value,
+			     GParamSpec   *pspec)
+{
+	GeditNotebook *notebook = GEDIT_NOTEBOOK (object);
+
+	switch (prop_id)
+	{
+		case PROP_SHOW_TABS_MODE:
+			notebook->priv->show_tabs_mode = g_value_get_enum (value);
+			update_tabs_visibility (notebook, FALSE);
+			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
+	}
+}
+
+static void
+gedit_notebook_dispose (GObject *object)
+{
+	GeditNotebook *notebook = GEDIT_NOTEBOOK (object);
+
+	if (notebook->priv->ui_settings != NULL)
+	{
+		g_object_unref (notebook->priv->ui_settings);
+		notebook->priv->ui_settings = NULL;
+	}
+
+	G_OBJECT_CLASS (gedit_notebook_parent_class)->dispose (object);
+}
+
+static void
+gedit_notebook_finalize (GObject *object)
+{
+	GeditNotebook *notebook = GEDIT_NOTEBOOK (object);
+
+	g_list_free (notebook->priv->focused_pages);
+
+	G_OBJECT_CLASS (gedit_notebook_parent_class)->finalize (object);
+}
+
+static void
+gedit_notebook_grab_focus (GtkWidget *widget)
+{
+	GtkNotebook *nb = GTK_NOTEBOOK (widget);
+	GtkWidget *tab;
+	gint current_page;
+
+	current_page = gtk_notebook_get_current_page (nb);
+	tab = gtk_notebook_get_nth_page (nb, current_page);
+
+	gtk_widget_grab_focus (tab);
+}
+
+static void
+gedit_notebook_style_set (GtkWidget *widget,
+			  GtkStyle  *previous_style)
+{
+	GeditNotebook *nb = GEDIT_NOTEBOOK (widget);
+
+	if (previous_style != NULL && nb->priv->collapse_border)
+	{
+		previous_style->xthickness = 0;
+	}
+
+	GTK_WIDGET_CLASS (gedit_notebook_parent_class)->style_set (widget, previous_style);
+}
+
 static void
 gedit_notebook_class_init (GeditNotebookClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *gtkwidget_class = GTK_WIDGET_CLASS (klass);
 	GtkNotebookClass *notebook_class = GTK_NOTEBOOK_CLASS (klass);
 
+	object_class->dispose = gedit_notebook_dispose;
 	object_class->finalize = gedit_notebook_finalize;
-	
+	object_class->get_property = gedit_notebook_get_property;
+	object_class->set_property = gedit_notebook_set_property;
+
+	gtkwidget_class->grab_focus = gedit_notebook_grab_focus;
+	gtkwidget_class->style_set = gedit_notebook_style_set;
+
 	notebook_class->change_current_page = gedit_notebook_change_current_page;
 
-	signals[TAB_ADDED] =
-		g_signal_new ("tab_added",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (GeditNotebookClass, tab_added),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1,
-			      GEDIT_TYPE_TAB);
-	signals[TAB_REMOVED] =
-		g_signal_new ("tab_removed",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (GeditNotebookClass, tab_removed),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1,
-			      GEDIT_TYPE_TAB);
+	g_object_class_install_property (object_class, PROP_SHOW_TABS_MODE,
+					 g_param_spec_enum ("show-tabs-mode",
+							    "Show Tabs Mode",
+							    "When tabs should be shown",
+							    GEDIT_TYPE_NOTEBOOK_SHOW_TABS_MODE_TYPE,
+							    GEDIT_NOTEBOOK_SHOW_TABS_ALWAYS,
+							    G_PARAM_READWRITE));
+
 	signals[TAB_DETACHED] =
 		g_signal_new ("tab_detached",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -158,15 +287,36 @@ gedit_notebook_class_init (GeditNotebookClass *klass)
 			      1,
 			      GEDIT_TYPE_TAB);
 
-	g_type_class_add_private (object_class, sizeof(GeditNotebookPrivate));
+	g_type_class_add_private (object_class, sizeof (GeditNotebookPrivate));
 }
 
 static GeditNotebook *
-find_notebook_at_pointer (gint abs_x, gint abs_y)
+get_notebook_from_widget (GtkWidget *widget)
+{
+	if (widget == NULL)
+	{
+		return NULL;
+	}
+
+	while (!gtk_widget_is_toplevel (GTK_WIDGET (widget)))
+	{
+		if (GEDIT_IS_NOTEBOOK (widget))
+		{
+			return GEDIT_NOTEBOOK (widget);
+		}
+
+		widget = gtk_widget_get_parent (GTK_WIDGET (widget));
+	}
+
+	return NULL;
+}
+
+static GeditNotebook *
+find_notebook_at_pointer (gint abs_x,
+			  gint abs_y)
 {
 	GdkWindow *win_at_pointer;
-	GdkWindow *toplevel_win;
-	gpointer toplevel = NULL;
+	gpointer widget = NULL;
 	gint x, y;
 
 	/* FIXME multi-head */
@@ -177,21 +327,10 @@ find_notebook_at_pointer (gint abs_x, gint abs_y)
 		return NULL;
 	}
 
-	toplevel_win = gdk_window_get_toplevel (win_at_pointer);
-
 	/* get the GtkWidget which owns the toplevel GdkWindow */
-	gdk_window_get_user_data (toplevel_win, &toplevel);
+	gdk_window_get_user_data (win_at_pointer, &widget);
 
-	/* toplevel should be an GeditWindow */
-	if ((toplevel != NULL) && 
-	    GEDIT_IS_WINDOW (toplevel))
-	{
-		return GEDIT_NOTEBOOK (_gedit_window_get_notebook
-						(GEDIT_WINDOW (toplevel)));
-	}
-
-	/* We are outside all windows containing a notebook */
-	return NULL;
+	return get_notebook_from_widget (widget);
 }
 
 static gboolean
@@ -220,11 +359,6 @@ find_tab_num_at_pos (GeditNotebook *notebook,
 
 	tab_pos = gtk_notebook_get_tab_pos (GTK_NOTEBOOK (notebook));
 
-	if (GTK_NOTEBOOK (notebook)->first_tab == NULL)
-	{
-		return AFTER_ALL_TABS;
-	}
-
 	/* For some reason unfullscreen + quick click can
 	   cause a wrong click event to be reported to the tab */
 	if (!is_in_notebook_window (notebook, abs_x, abs_y))
@@ -237,21 +371,23 @@ find_tab_num_at_pos (GeditNotebook *notebook,
 		GtkWidget *tab;
 		gint max_x, max_y;
 		gint x_root, y_root;
+		GtkAllocation allocation;
 
 		tab = gtk_notebook_get_tab_label (nb, page);
 		g_return_val_if_fail (tab != NULL, AFTER_ALL_TABS);
 
-		if (!GTK_WIDGET_MAPPED (GTK_WIDGET (tab)))
+		if (!gtk_widget_get_mapped (tab))
 		{
 			++page_num;
 			continue;
 		}
 
-		gdk_window_get_origin (GDK_WINDOW (tab->window),
+		gdk_window_get_origin (GDK_WINDOW (gtk_widget_get_window (tab)),
 				       &x_root, &y_root);
 
-		max_x = x_root + tab->allocation.x + tab->allocation.width;
-		max_y = y_root + tab->allocation.y + tab->allocation.height;
+		gtk_widget_get_allocation (tab, &allocation);
+		max_x = x_root + allocation.x + allocation.width;
+		max_y = y_root + allocation.y + allocation.height;
 
 		if (((tab_pos == GTK_POS_TOP) || 
 		     (tab_pos == GTK_POS_BOTTOM)) &&
@@ -314,7 +450,7 @@ gedit_notebook_move_tab (GeditNotebook *src,
 			 GeditTab      *tab,
 			 gint           dest_position)
 {
-	g_return_if_fail (GEDIT_IS_NOTEBOOK (src));	
+	g_return_if_fail (GEDIT_IS_NOTEBOOK (src));
 	g_return_if_fail (GEDIT_IS_NOTEBOOK (dest));
 	g_return_if_fail (src != dest);
 	g_return_if_fail (GEDIT_IS_TAB (tab));
@@ -630,14 +766,13 @@ gedit_notebook_new (void)
 }
 
 static void
-gedit_notebook_switch_page_cb (GtkNotebook     *notebook,
-                               GtkNotebookPage *page,
-                               guint            page_num,
-                               gpointer         data)
+gedit_notebook_switch_page_cb (GtkNotebook *notebook,
+                               GtkWidget   *page,
+                               guint        page_num,
+                               gpointer     data)
 {
 	GeditNotebook *nb = GEDIT_NOTEBOOK (notebook);
 	GtkWidget *child;
-	GeditView *view;
 
 	child = gtk_notebook_get_nth_page (notebook, page_num);
 
@@ -652,29 +787,8 @@ gedit_notebook_switch_page_cb (GtkNotebook     *notebook,
 	nb->priv->focused_pages = g_list_append (nb->priv->focused_pages,
 						 child);
 
-	/* give focus to the view */
-	view = gedit_tab_get_view (GEDIT_TAB (child));
-	gtk_widget_grab_focus (GTK_WIDGET (view));
-}
-
-/*
- * update_tabs_visibility: Hide tabs if there is only one tab
- * and the pref is not set.
- */
-static void
-update_tabs_visibility (GeditNotebook *nb, 
-			gboolean       before_inserting)
-{
-	gboolean show_tabs;
-	guint num;
-
-	num = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb));
-
-	if (before_inserting) num++;
-
-	show_tabs = (nb->priv->always_show_tabs || num > 1);
-
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (nb), show_tabs);
+	/* give focus to the tab */
+	gtk_widget_grab_focus (child);
 }
 
 static void
@@ -682,43 +796,40 @@ gedit_notebook_init (GeditNotebook *notebook)
 {
 	notebook->priv = GEDIT_NOTEBOOK_GET_PRIVATE (notebook);
 
+	notebook->priv->ui_settings = g_settings_new ("org.gnome.gedit.preferences.ui");
+
+	notebook->priv->show_tabs_mode = GEDIT_NOTEBOOK_SHOW_TABS_ALWAYS;
 	notebook->priv->close_buttons_sensitive = TRUE;
 	notebook->priv->tab_drag_and_drop_enabled = TRUE;
 	
 	gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
+	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), TRUE);
+	gtk_container_set_border_width (GTK_CONTAINER (notebook), 0);
 
-	notebook->priv->always_show_tabs = TRUE;
+	g_settings_bind (notebook->priv->ui_settings,
+			 GEDIT_SETTINGS_SHOW_TABS_MODE,
+			 notebook,
+			 "show-tabs-mode",
+			 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
 
-	g_signal_connect (notebook, 
+	g_signal_connect (notebook,
 			  "button-press-event",
-			  (GCallback)button_press_cb, 
+			  (GCallback)button_press_cb,
 			  NULL);
-	g_signal_connect (notebook, 
+	g_signal_connect (notebook,
 			  "button-release-event",
 			  (GCallback)button_release_cb,
 			  NULL);
-	gtk_widget_add_events (GTK_WIDGET (notebook), 
+	gtk_widget_add_events (GTK_WIDGET (notebook),
 			       GDK_BUTTON1_MOTION_MASK);
 
-	g_signal_connect_after (G_OBJECT (notebook), 
+	g_signal_connect_after (G_OBJECT (notebook),
 				"switch_page",
                                 G_CALLBACK (gedit_notebook_switch_page_cb),
                                 NULL);
 }
 
-static void
-gedit_notebook_finalize (GObject *object)
-{
-	GeditNotebook *notebook = GEDIT_NOTEBOOK (object);
-
-	if (notebook->priv->focused_pages)
-		g_list_free (notebook->priv->focused_pages);
-
-	G_OBJECT_CLASS (gedit_notebook_parent_class)->finalize (object);
-}
-
 /*
  * We need to override this because when we don't show the tabs, like in
  * fullscreen we need to have wrap around too
@@ -815,24 +926,6 @@ get_tab_label (GeditTab *tab)
 	return tab_label;
 }
 
-/**
- * gedit_notebook_set_always_show_tabs:
- * @nb: a #GeditNotebook
- * @show_tabs: %TRUE to always show the tabs
- *
- * Sets the visibility of the tabs in the @nb.
- */
-void
-gedit_notebook_set_always_show_tabs (GeditNotebook *nb, 
-				     gboolean       show_tabs)
-{
-	g_return_if_fail (GEDIT_IS_NOTEBOOK (nb));
-
-	nb->priv->always_show_tabs = (show_tabs != FALSE);
-
-	update_tabs_visibility (nb, FALSE);
-}
-
 /**
  * gedit_notebook_add_tab:
  * @nb: a #GeditNotebook
@@ -854,29 +947,26 @@ gedit_notebook_add_tab (GeditNotebook *nb,
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 
 	tab_label = create_tab_label (nb, tab);
+
 	gtk_notebook_insert_page (GTK_NOTEBOOK (nb), 
 				  GTK_WIDGET (tab),
 				  tab_label,
 				  position);
-	update_tabs_visibility (nb, TRUE);
 
-	g_signal_emit (G_OBJECT (nb), signals[TAB_ADDED], 0, tab);
+	update_tabs_visibility (nb, FALSE);
 
 	/* The signal handler may have reordered the tabs */
-	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), 
+	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
 					  GTK_WIDGET (tab));
 
 	if (jump_to)
 	{
-		GeditView *view;
-		
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb), position);
 		g_object_set_data (G_OBJECT (tab), 
 				   "jump_to",
 				   GINT_TO_POINTER (jump_to));
-		view = gedit_tab_get_view (tab);
 		
-		gtk_widget_grab_focus (GTK_WIDGET (view));
+		gtk_widget_grab_focus (GTK_WIDGET (tab));
 	}
 }
 
@@ -914,21 +1004,17 @@ remove_tab (GeditTab      *tab,
 	    GeditNotebook *nb)
 {
 	gint position;
+	gint num_pages;
 
 	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
 
-	/* we ref the tab so that it's still alive while the tabs_removed
-	 * signal is processed.
-	 */
-	g_object_ref (tab);
-
 	remove_tab_label (nb, tab);
+	num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb));
 	gtk_notebook_remove_page (GTK_NOTEBOOK (nb), position);
-	update_tabs_visibility (nb, FALSE);
 
-	g_signal_emit (G_OBJECT (nb), signals[TAB_REMOVED], 0, tab);
-
-	g_object_unref (tab);
+	/* If there is no tabs, calling this is pointless */
+	if (--num_pages > 0)
+		update_tabs_visibility (nb, FALSE);
 }
 
 /**
@@ -1071,3 +1157,34 @@ gedit_notebook_get_tab_drag_and_drop_enabled (GeditNotebook *nb)
 	return nb->priv->tab_drag_and_drop_enabled;
 }
 
+void
+gedit_notebook_collapse_border (GeditNotebook *nb,
+				gboolean       collapse)
+{
+	GtkRcStyle *rcstyle;
+	gint xthickness;
+
+	g_return_if_fail (GEDIT_IS_NOTEBOOK (nb));
+
+	nb->priv->collapse_border = collapse;
+
+	if (collapse)
+	{
+		GtkStyle *style;
+
+		style = gtk_widget_get_style (GTK_WIDGET (nb));
+		nb->priv->xthickness = style->xthickness;
+		xthickness = 0;
+	}
+	else
+	{
+		xthickness = nb->priv->xthickness;
+	}
+
+	rcstyle = gtk_rc_style_new ();
+	rcstyle->xthickness = xthickness;
+	gtk_widget_modify_style (GTK_WIDGET (nb), rcstyle);
+	g_object_unref (rcstyle);
+}
+
+/* ex:ts=8:noet: */
