@@ -331,7 +331,7 @@ on_undo_actions_limit_changed (GSettings     *settings,
 	GList *docs, *l;
 	gint ul;
 	
-	g_settings_get (settings, key, "u", &ul);
+	ul = g_settings_get_int (settings, key);
 	
 	ul = CLAMP (ul, -1, 250);
 	
@@ -354,7 +354,7 @@ on_wrap_mode_changed (GSettings     *settings,
 	GtkWrapMode wrap_mode;
 	GList *views, *l;
 
-	wrap_mode = gedit_settings_get_wrap_mode (settings, key);
+	wrap_mode = g_settings_get_enum (settings, key);
 
 	views = gedit_app_get_views (gedit_app_get_default ());
 	
@@ -550,7 +550,7 @@ on_smart_home_end_changed (GSettings     *settings,
 	GtkSourceSmartHomeEndType smart_he;
 	GList *views, *l;
 
-	smart_he = gedit_settings_get_smart_home_end (gs);
+	smart_he = g_settings_get_enum (settings, key);
 
 	views = gedit_app_get_views (gedit_app_get_default ());
 	
@@ -648,18 +648,6 @@ on_max_recents_changed (GSettings     *settings,
 	 * update of the inline recents in the File menu */
 }
 
-static void
-on_active_plugins_changed (GSettings     *settings,
-			   const gchar   *key,
-			   GeditSettings *gs)
-{
-	GeditPluginsEngine *engine;
-
-	engine = gedit_plugins_engine_get_default ();
-
-	gedit_plugins_engine_active_plugins_changed (engine);
-}
-
 static void
 gedit_settings_init (GeditSettings *gs)
 {
@@ -680,17 +668,17 @@ initialize (GeditSettings *gs)
 	gs->priv->plugins = g_settings_get_child (G_SETTINGS (gs), "plugins");
 	
 	/* Load settings */
-	gs->priv->lockdown = g_settings_new ("org.gnome.Desktop.Lockdown");
+	gs->priv->lockdown = g_settings_new ("org.gnome.desktop.lockdown");
 	
 	g_signal_connect (gs->priv->lockdown,
 			  "changed",
 			  G_CALLBACK (on_lockdown_changed),
 			  NULL);
 
-	gs->priv->interface = g_settings_new ("org.gnome.Desktop.Interface");
+	gs->priv->interface = g_settings_new ("org.gnome.desktop.interface");
 	
 	g_signal_connect (gs->priv->interface,
-			  "changed",
+			  "changed::monospace-font-name",
 			  G_CALLBACK (on_system_font_changed),
 			  gs);
 
@@ -773,12 +761,6 @@ initialize (GeditSettings *gs)
 			  "changed::max-recents",
 			  G_CALLBACK (on_max_recents_changed),
 			  gs);
-
-	/* plugins changes */
-	g_signal_connect (gs->priv->plugins,
-			  "changed::active-plugins",
-			  G_CALLBACK (on_active_plugins_changed),
-			  gs);
 }
 
 static void
@@ -855,15 +837,15 @@ gedit_settings_get_list (GSettings   *settings,
 {
 	GSList *list = NULL;
 	gchar **values;
-	gsize len, i;
+	gsize i;
 
 	g_return_val_if_fail (G_IS_SETTINGS (settings), NULL);
 	g_return_val_if_fail (key != NULL, NULL);
 
-	values = g_settings_get_strv (settings, key, &len);
+	values = g_settings_get_strv (settings, key);
 	i = 0;
 
-	while (i < len)
+	while (values[i] != NULL)
 	{
 		list = g_slist_prepend (list, values[i]);
 		i++;
@@ -881,155 +863,26 @@ gedit_settings_set_list (GSettings    *settings,
 {
 	gchar **values = NULL;
 	const GSList *l;
-	gint len = 0;
 
 	g_return_if_fail (G_IS_SETTINGS (settings));
 	g_return_if_fail (key != NULL);
 
 	if (list != NULL)
 	{
-		gint i;
+		gint i, len;
 
 		len = g_slist_length ((GSList *)list);
-		values = g_new (gchar *, len);
+		values = g_new (gchar *, len + 1);
 
 		for (l = list, i = 0; l != NULL; l = g_slist_next (l), i++)
 		{
 			values[i] = l->data;
 		}
+		values[i] = NULL;
 	}
 
-	g_settings_set_strv (settings, key, (const gchar * const *)values, len);
+	g_settings_set_strv (settings, key, (const gchar * const *)values);
 	g_free (values);
 }
 
-GeditToolbarSetting
-gedit_settings_get_toolbar_style (GeditSettings *gs)
-{
-	GeditToolbarSetting res;
-	gchar *str;
-
-	g_return_val_if_fail (GEDIT_IS_SETTINGS (gs), 0);
-
-	str = g_settings_get_string (gs->priv->ui,
-				     "toolbar-buttons-style");
-
-	g_return_val_if_fail (str != NULL, 0);
-
-	if (strcmp (str, "GEDIT_TOOLBAR_ICONS") == 0)
-		res = GEDIT_TOOLBAR_ICONS;
-	else
-	{
-		if (strcmp (str, "GEDIT_TOOLBAR_ICONS_AND_TEXT") == 0)
-		{
-			res = GEDIT_TOOLBAR_ICONS_AND_TEXT;
-		}
-		else 
-		{
-			if (strcmp (str, "GEDIT_TOOLBAR_ICONS_BOTH_HORIZ") == 0)
-				res = GEDIT_TOOLBAR_ICONS_BOTH_HORIZ;
-			else
-				res = GEDIT_TOOLBAR_SYSTEM;
-		}
-	}
-
-	g_free (str);
-
-	return res;
-}
-
-GtkSourceSmartHomeEndType
-gedit_settings_get_smart_home_end (GeditSettings *gs)
-{
-	GtkSourceSmartHomeEndType res;
-	gchar *str;
-	
-	g_return_val_if_fail (GEDIT_IS_SETTINGS (gs), GTK_SOURCE_SMART_HOME_END_AFTER);
-	
-	str = g_settings_get_string (gs->priv->editor, "smart-home-end");
-
-	g_return_val_if_fail (str != NULL, GTK_SOURCE_SMART_HOME_END_AFTER);
-
-	if (strcmp (str, "DISABLED") == 0)
-		res = GTK_SOURCE_SMART_HOME_END_DISABLED;
-	else if (strcmp (str, "BEFORE") == 0)
-		res = GTK_SOURCE_SMART_HOME_END_BEFORE;
-	else if (strcmp (str, "ALWAYS") == 0)
-		res = GTK_SOURCE_SMART_HOME_END_ALWAYS;
-	else
-		res = GTK_SOURCE_SMART_HOME_END_AFTER;
-
-	g_free (str);
-
-	return res;
-}
-
-static gchar *
-get_wrap_str (guint mode)
-{
-	gchar *str;
-
-	switch (mode)
-	{
-		case GTK_WRAP_NONE:
-			str = g_strdup ("GTK_WRAP_NONE");
-			break;
-
-		case GTK_WRAP_CHAR:
-			str = g_strdup ("GTK_WRAP_CHAR");
-			break;
-
-		default: /* GTK_WRAP_WORD */
-			str = g_strdup ("GTK_WRAP_WORD");
-	}
-	
-	return str;
-}
-
-GtkWrapMode
-gedit_settings_get_wrap_mode (GSettings   *settings,
-			      const gchar *key)
-{
-	GtkWrapMode res;
-	gchar *str;
-
-	g_return_val_if_fail (G_IS_SETTINGS (settings), GTK_WRAP_WORD);
-	g_return_val_if_fail (key != NULL, GTK_WRAP_WORD);
-
-	str = g_settings_get_string (settings, key);
-
-	g_return_val_if_fail (str != NULL, GTK_WRAP_WORD);
-
-	if (strcmp (str, "GTK_WRAP_NONE") == 0)
-	{
-		res = GTK_WRAP_NONE;
-	}
-	else
-	{
-		if (strcmp (str, "GTK_WRAP_CHAR") == 0)
-			res = GTK_WRAP_CHAR;
-		else
-			res = GTK_WRAP_WORD;
-	}
-
-	g_free (str);
-
-	return res;
-}
-
-void
-gedit_settings_set_wrap_mode (GSettings   *settings,
-			      const gchar *key,
-			      GtkWrapMode  mode)
-{
-	gchar *str;
-
-	g_return_if_fail (G_IS_SETTINGS (settings));
-	g_return_if_fail (key != NULL);
-
-	str = get_wrap_str (mode);
-
-	g_settings_set_string (settings, key, str);
-}
-
 /* ex:ts=8:noet: */
