@@ -83,10 +83,6 @@ static const GtkActionEntry gedit_menu_entries[] =
 	  N_("Save the current file with a different name"), G_CALLBACK (_gedit_cmd_file_save_as) },
 	{ "FileRevert", GTK_STOCK_REVERT_TO_SAVED, NULL, NULL,
 	  N_("Revert to a saved version of the file"), G_CALLBACK (_gedit_cmd_file_revert) },
-#if !GTK_CHECK_VERSION (2, 17, 4)
-	{ "FilePageSetup", GTK_STOCK_PAGE_SETUP, N_("Page Set_up..."), NULL,
-	  N_("Set up the page settings"), G_CALLBACK (_gedit_cmd_file_page_setup) },
-#endif
 	{ "FilePrintPreview", GTK_STOCK_PRINT_PREVIEW, N_("Print Previe_w"),"<control><shift>P",
 	  N_("Print preview"), G_CALLBACK (_gedit_cmd_file_print_preview) },
 	 { "FilePrint", GTK_STOCK_PRINT, N_("_Print..."), "<control>P",
@@ -178,15 +174,15 @@ static const GtkToggleActionEntry gedit_always_sensitive_toggle_menu_entries[] =
 	  G_CALLBACK (_gedit_cmd_view_toggle_fullscreen_mode), FALSE }
 };
 
-/* separate group, should be always sensitive except when there are no panes */
-static const GtkToggleActionEntry gedit_panes_toggle_menu_entries[] =
+/* separate group, should be always sensitive except when there are no panels */
+static const GtkToggleActionEntry gedit_panels_toggle_menu_entries[] =
 {
-	{ "ViewSidePane", NULL, N_("Side _Pane"), "F9",
-	  N_("Show or hide the side pane in the current window"),
-	  G_CALLBACK (_gedit_cmd_view_show_side_pane), FALSE },
-	{ "ViewBottomPane", NULL, N_("_Bottom Pane"), "<control>F9",
-	  N_("Show or hide the bottom pane in the current window"),
-	  G_CALLBACK (_gedit_cmd_view_show_bottom_pane), FALSE }
+	{ "ViewSidePanel", NULL, N_("Side _Panel"), "F9",
+	  N_("Show or hide the side panel in the current window"),
+	  G_CALLBACK (_gedit_cmd_view_show_side_panel), FALSE },
+	{ "ViewBottomPanel", NULL, N_("_Bottom Panel"), "<control>F9",
+	  N_("Show or hide the bottom panel in the current window"),
+	  G_CALLBACK (_gedit_cmd_view_show_bottom_panel), FALSE }
 };
 
 G_END_DECLS
