@@ -16,8 +16,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * $Id$
  */
 
 /*
@@ -38,16 +36,17 @@
 
 #include <glib/gi18n-lib.h>
 #include <glib.h>
-#include <gmodule.h>
 
 #include <gedit/gedit-debug.h>
 #include <gedit/gedit-utils.h>
+#include <gedit/gedit-window.h>
+#include <gedit/gedit-window-activatable.h>
+#include <libpeas-gtk/peas-gtk-configurable.h>
 
 #define GEDIT_TIME_PLUGIN_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), \
 					      GEDIT_TYPE_TIME_PLUGIN, \
 					      GeditTimePluginPrivate))
 
-#define WINDOW_DATA_KEY "GeditTimePluginWindowData"
 #define MENU_PATH "/MenuBar/EditMenu/EditOps_4"
 
 /* gsettings keys */
@@ -103,24 +102,31 @@ enum
 	NUM_COLUMNS
 };
 
-typedef struct _TimeConfigureDialog TimeConfigureDialog;
-
-struct _TimeConfigureDialog
+typedef enum
 {
-	GtkWidget *dialog;
+	PROMPT_SELECTED_FORMAT = 0,	/* Popup dialog with list preselected */
+	PROMPT_CUSTOM_FORMAT,		/* Popup dialog with entry preselected */
+	USE_SELECTED_FORMAT,		/* Use selected format directly */
+	USE_CUSTOM_FORMAT		/* Use custom format directly */
+} GeditTimePluginPromptType;
+
+typedef struct _TimeConfigureWidget TimeConfigureWidget;
+
+struct _TimeConfigureWidget
+{
+	GtkWidget *content;
 
 	GtkWidget *list;
 
-        /* Radio buttons to indicate what should be done */
-        GtkWidget *prompt;
-        GtkWidget *use_list;
-        GtkWidget *custom;
+	/* Radio buttons to indicate what should be done */
+	GtkWidget *prompt;
+	GtkWidget *use_list;
+	GtkWidget *custom;
 
 	GtkWidget *custom_entry;
 	GtkWidget *custom_format_example;
 
-	/* Info needed for the response handler */
-	GeditTimePlugin *plugin;
+	GSettings *settings;
 };
 
 typedef struct _ChooseFormatDialog ChooseFormatDialog;
@@ -131,46 +137,48 @@ struct _ChooseFormatDialog
 
 	GtkWidget *list;
 
-        /* Radio buttons to indicate what should be done */
-        GtkWidget *use_list;
-        GtkWidget *custom;
+	/* Radio buttons to indicate what should be done */
+	GtkWidget *use_list;
+	GtkWidget *custom;
 
-        GtkWidget *custom_entry;
+	GtkWidget *custom_entry;
 	GtkWidget *custom_format_example;
 
 	/* Info needed for the response handler */
 	GtkTextBuffer   *buffer;
-	GeditTimePlugin *plugin;
-};
 
-typedef enum
-{
-	PROMPT_SELECTED_FORMAT = 0,	/* Popup dialog with list preselected */
-	PROMPT_CUSTOM_FORMAT,		/* Popup dialog with entry preselected */
-	USE_SELECTED_FORMAT,		/* Use selected format directly */
-	USE_CUSTOM_FORMAT		/* Use custom format directly */
-} GeditTimePluginPromptType;
-
-struct _GeditTimePluginPrivate
-{
 	GSettings *settings;
 };
 
-GEDIT_PLUGIN_REGISTER_TYPE(GeditTimePlugin, gedit_time_plugin)
-
-typedef struct
+struct _GeditTimePluginPrivate
 {
+	GSettings      *settings;
+
+	GeditWindow    *window;
+
 	GtkActionGroup *action_group;
 	guint           ui_id;
-} WindowData;
+};
 
-typedef struct
+enum
 {
-	GeditWindow     *window;
-	GeditTimePlugin *plugin;
-} ActionData;
+	PROP_0,
+	PROP_WINDOW
+};
 
-static void time_cb (GtkAction *action, ActionData *data);
+static void gedit_window_activatable_iface_init (GeditWindowActivatableInterface *iface);
+static void peas_gtk_configurable_iface_init (PeasGtkConfigurableInterface *iface);
+
+G_DEFINE_DYNAMIC_TYPE_EXTENDED (GeditTimePlugin,
+				gedit_time_plugin,
+				PEAS_TYPE_EXTENSION_BASE,
+				0,
+				G_IMPLEMENT_INTERFACE_DYNAMIC (GEDIT_TYPE_WINDOW_ACTIVATABLE,
+							       gedit_window_activatable_iface_init)
+				G_IMPLEMENT_INTERFACE_DYNAMIC (PEAS_GTK_TYPE_CONFIGURABLE,
+							       peas_gtk_configurable_iface_init))
+
+static void time_cb (GtkAction *action, GeditTimePlugin *plugin);
 
 static const GtkActionEntry action_entries[] =
 {
@@ -207,40 +215,74 @@ gedit_time_plugin_dispose (GObject *object)
 		plugin->priv->settings = NULL;
 	}
 
+	if (plugin->priv->action_group != NULL)
+	{
+		g_object_unref (plugin->priv->action_group);
+		plugin->priv->action_group = NULL;
+	}
+
+	if (plugin->priv->window != NULL)
+	{
+		g_object_unref (plugin->priv->window);
+		plugin->priv->window = NULL;
+	}
+
 	G_OBJECT_CLASS (gedit_time_plugin_parent_class)->dispose (object);
 }
 
 static void
-free_window_data (WindowData *data)
+gedit_time_plugin_set_property (GObject      *object,
+                                guint         prop_id,
+                                const GValue *value,
+                                GParamSpec   *pspec)
 {
-	g_return_if_fail (data != NULL);
+	GeditTimePlugin *plugin = GEDIT_TIME_PLUGIN (object);
 
-	g_object_unref (data->action_group);
-	g_slice_free (WindowData, data);
+	switch (prop_id)
+	{
+		case PROP_WINDOW:
+			plugin->priv->window = GEDIT_WINDOW (g_value_dup_object (value));
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
+	}
 }
 
 static void
-free_action_data (ActionData *data)
+gedit_time_plugin_get_property (GObject    *object,
+                                guint       prop_id,
+                                GValue     *value,
+                                GParamSpec *pspec)
 {
-	g_return_if_fail (data != NULL);
+	GeditTimePlugin *plugin = GEDIT_TIME_PLUGIN (object);
 
-	g_slice_free (ActionData, data);
+	switch (prop_id)
+	{
+		case PROP_WINDOW:
+			g_value_set_object (value, plugin->priv->window);
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
+	}
 }
 
 static void
-update_ui_real (GeditWindow  *window,
-		WindowData   *data)
+update_ui (GeditTimePlugin *plugin)
 {
 	GeditView *view;
 	GtkAction *action;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	view = gedit_window_get_active_view (window);
+	view = gedit_window_get_active_view (plugin->priv->window);
 
 	gedit_debug_message (DEBUG_PLUGINS, "View: %p", view);
 
-	action = gtk_action_group_get_action (data->action_group,
+	action = gtk_action_group_get_action (plugin->priv->action_group,
 					      "InsertDateAndTime");
 	gtk_action_set_sensitive (action,
 				  (view != NULL) &&
@@ -248,137 +290,62 @@ update_ui_real (GeditWindow  *window,
 }
 
 static void
-impl_activate (GeditPlugin *plugin,
-	       GeditWindow *window)
+gedit_time_plugin_activate (GeditWindowActivatable *activatable)
 {
+	GeditTimePluginPrivate *priv;
 	GtkUIManager *manager;
-	WindowData *data;
-	ActionData *action_data;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	data = g_slice_new (WindowData);
-	action_data = g_slice_new (ActionData);
+	priv = GEDIT_TIME_PLUGIN (activatable)->priv;
 
-	action_data->plugin = GEDIT_TIME_PLUGIN (plugin);
-	action_data->window = window;
+	manager = gedit_window_get_ui_manager (priv->window);
 
-	manager = gedit_window_get_ui_manager (window);
-
-	data->action_group = gtk_action_group_new ("GeditTimePluginActions");
-	gtk_action_group_set_translation_domain (data->action_group,
+	priv->action_group = gtk_action_group_new ("GeditTimePluginActions");
+	gtk_action_group_set_translation_domain (priv->action_group,
 						 GETTEXT_PACKAGE);
-	gtk_action_group_add_actions_full (data->action_group,
-				      	   action_entries,
-				      	   G_N_ELEMENTS (action_entries),
-				      	   action_data,
-				      	   (GDestroyNotify) free_action_data);
+	gtk_action_group_add_actions (priv->action_group,
+				      action_entries,
+				      G_N_ELEMENTS (action_entries),
+				      activatable);
 
-	gtk_ui_manager_insert_action_group (manager, data->action_group, -1);
+	gtk_ui_manager_insert_action_group (manager, priv->action_group, -1);
 
-	data->ui_id = gtk_ui_manager_new_merge_id (manager);
-
-	g_object_set_data_full (G_OBJECT (window),
-				WINDOW_DATA_KEY,
-				data,
-				(GDestroyNotify) free_window_data);
+	priv->ui_id = gtk_ui_manager_new_merge_id (manager);
 
 	gtk_ui_manager_add_ui (manager,
-			       data->ui_id,
+			       priv->ui_id,
 			       MENU_PATH,
 			       "InsertDateAndTime",
 			       "InsertDateAndTime",
 			       GTK_UI_MANAGER_MENUITEM,
 			       FALSE);
 
-	update_ui_real (window, data);
+	update_ui (GEDIT_TIME_PLUGIN (activatable));
 }
 
 static void
-impl_deactivate	(GeditPlugin *plugin,
-		 GeditWindow *window)
+gedit_time_plugin_deactivate (GeditWindowActivatable *activatable)
 {
+	GeditTimePluginPrivate *priv;
 	GtkUIManager *manager;
-	WindowData *data;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	manager = gedit_window_get_ui_manager (window);
+	priv = GEDIT_TIME_PLUGIN (activatable)->priv;
 
-	data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
-	g_return_if_fail (data != NULL);
+	manager = gedit_window_get_ui_manager (priv->window);
 
-	gtk_ui_manager_remove_ui (manager, data->ui_id);
-	gtk_ui_manager_remove_action_group (manager, data->action_group);
-
-	g_object_set_data (G_OBJECT (window), WINDOW_DATA_KEY, NULL);
+	gtk_ui_manager_remove_ui (manager, priv->ui_id);
+	gtk_ui_manager_remove_action_group (manager, priv->action_group);
 }
 
 static void
-impl_update_ui	(GeditPlugin *plugin,
-		 GeditWindow *window)
+gedit_time_plugin_update_state (GeditWindowActivatable *activatable)
 {
-	WindowData   *data;
-
 	gedit_debug (DEBUG_PLUGINS);
 
-	data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
-	g_return_if_fail (data != NULL);
-
-	update_ui_real (window, data);
-}
-
-/* whether we should prompt the user or use the specified format */
-static GeditTimePluginPromptType
-get_prompt_type (GeditTimePlugin *plugin)
-{
-	gchar *prompt_type;
-	GeditTimePluginPromptType res;
-
-	prompt_type = g_settings_get_string (plugin->priv->settings,
-			        	     PROMPT_TYPE_KEY);
-
-	if (prompt_type == NULL)
-		return PROMPT_SELECTED_FORMAT;
-
-	if (strcmp (prompt_type, "USE_SELECTED_FORMAT") == 0)
-		res = USE_SELECTED_FORMAT;
-	else if (strcmp (prompt_type, "USE_CUSTOM_FORMAT") == 0)
-		res = USE_CUSTOM_FORMAT;
-	else if (strcmp (prompt_type, "PROMPT_CUSTOM_FORMAT") == 0)
-		res = PROMPT_CUSTOM_FORMAT;
-	else
-		res = PROMPT_SELECTED_FORMAT;
-
-	g_free (prompt_type);
-
-	return res;
-}
-
-static void
-set_prompt_type (GeditTimePlugin           *plugin,
-		 GeditTimePluginPromptType  prompt_type)
-{
-	const gchar * str;
-
-	switch (prompt_type)
-	{
-		case USE_SELECTED_FORMAT:
-			str = "USE_SELECTED_FORMAT";
-			break;
-		case USE_CUSTOM_FORMAT:
-			str = "USE_CUSTOM_FORMAT";
-			break;
-		case PROMPT_CUSTOM_FORMAT:
-			str = "PROMPT_CUSTOM_FORMAT";
-			break;
-		default:
-			str = "PROMPT_SELECTED_FORMAT";
-	}
-
-	g_settings_set_string (plugin->priv->settings,
-			       PROMPT_TYPE_KEY,
-		       	       str);
+	update_ui (GEDIT_TIME_PLUGIN (activatable));
 }
 
 /* The selected format in the list */
@@ -393,17 +360,6 @@ get_selected_format (GeditTimePlugin *plugin)
 	return sel_format ? sel_format : g_strdup (formats [0]);
 }
 
-static void
-set_selected_format (GeditTimePlugin *plugin,
-		     const gchar     *format)
-{
-	g_return_if_fail (format != NULL);
-
-	g_settings_set_string (plugin->priv->settings,
-			       SELECTED_FORMAT_KEY,
-		       	       format);
-}
-
 /* the custom format in the entry */
 static gchar *
 get_custom_format (GeditTimePlugin *plugin)
@@ -416,25 +372,14 @@ get_custom_format (GeditTimePlugin *plugin)
 	return format ? format : g_strdup (DEFAULT_CUSTOM_FORMAT);
 }
 
-static void
-set_custom_format (GeditTimePlugin *plugin,
-		   const gchar     *format)
-{
-	g_return_if_fail (format != NULL);
-
-	g_settings_set_string (plugin->priv->settings,
-			       CUSTOM_FORMAT_KEY,
-		       	       format);
-}
-
 static gchar *
 get_time (const gchar* format)
 {
-  	gchar *out = NULL;
+	gchar *out = NULL;
 	gchar *out_utf8 = NULL;
-  	time_t clock;
-  	struct tm *now;
-  	size_t out_length = 0;
+	time_t clock;
+	struct tm *now;
+	size_t out_length = 0;
 	gchar *locale_format;
 
 	gedit_debug (DEBUG_PLUGINS);
@@ -448,15 +393,15 @@ get_time (const gchar* format)
 	if (locale_format == NULL)
 		return g_strdup (" ");
 
-  	clock = time (NULL);
-  	now = localtime (&clock);
+	clock = time (NULL);
+	now = localtime (&clock);
 
 	do
 	{
 		out_length += 255;
 		out = g_realloc (out, out_length);
 	}
-  	while (strftime (out, out_length, locale_format, now) == 0);
+	while (strftime (out, out_length, locale_format, now) == 0);
 
 	g_free (locale_format);
 
@@ -473,15 +418,30 @@ get_time (const gchar* format)
 			out_utf8 = g_strdup (" ");
 	}
 
-  	return out_utf8;
+	return out_utf8;
 }
 
 static void
-dialog_destroyed (GtkObject *obj, gpointer dialog_pointer)
+configure_widget_destroyed (GtkObject *obj,
+			    gpointer   data)
+{
+	TimeConfigureWidget *widget = (TimeConfigureWidget *)data;
+
+	gedit_debug (DEBUG_PLUGINS);
+
+	g_object_unref (widget->settings);
+	g_slice_free (TimeConfigureWidget, data);
+
+	gedit_debug_message (DEBUG_PLUGINS, "END");
+}
+
+static void
+choose_format_dialog_destroyed (GtkObject *obj,
+				gpointer   dialog_pointer)
 {
 	gedit_debug (DEBUG_PLUGINS);
 
-	g_free (dialog_pointer);
+	g_slice_free (ChooseFormatDialog, dialog_pointer);
 
 	gedit_debug_message (DEBUG_PLUGINS, "END");
 }
@@ -634,7 +594,7 @@ updated_custom_format_example (GtkEntry *format_entry,
 }
 
 static void
-choose_format_dialog_button_toggled (GtkToggleButton *button,
+choose_format_dialog_button_toggled (GtkToggleButton    *button,
 				     ChooseFormatDialog *dialog)
 {
 	gedit_debug (DEBUG_PLUGINS);
@@ -644,51 +604,59 @@ choose_format_dialog_button_toggled (GtkToggleButton *button,
 		gtk_widget_set_sensitive (dialog->list, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_entry, TRUE);
 		gtk_widget_set_sensitive (dialog->custom_format_example, TRUE);
-
-		return;
 	}
-
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->use_list)))
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->use_list)))
 	{
 		gtk_widget_set_sensitive (dialog->list, TRUE);
 		gtk_widget_set_sensitive (dialog->custom_entry, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_format_example, FALSE);
-
-		return;
+	}
+	else
+	{
+		g_return_if_reached ();
 	}
 }
 
 static void
-configure_dialog_button_toggled (GtkToggleButton *button, TimeConfigureDialog *dialog)
+configure_widget_button_toggled (GtkToggleButton     *button,
+				 TimeConfigureWidget *conf_widget)
 {
+	GeditTimePluginPromptType prompt_type;
+
 	gedit_debug (DEBUG_PLUGINS);
 
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->custom)))
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (conf_widget->custom)))
 	{
-		gtk_widget_set_sensitive (dialog->list, FALSE);
-		gtk_widget_set_sensitive (dialog->custom_entry, TRUE);
-		gtk_widget_set_sensitive (dialog->custom_format_example, TRUE);
+		gtk_widget_set_sensitive (conf_widget->list, FALSE);
+		gtk_widget_set_sensitive (conf_widget->custom_entry, TRUE);
+		gtk_widget_set_sensitive (conf_widget->custom_format_example, TRUE);
 
-		return;
+		prompt_type = USE_CUSTOM_FORMAT;
+	}
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (conf_widget->use_list)))
+	{
+		gtk_widget_set_sensitive (conf_widget->list, TRUE);
+		gtk_widget_set_sensitive (conf_widget->custom_entry, FALSE);
+		gtk_widget_set_sensitive (conf_widget->custom_format_example, FALSE);
+
+		prompt_type = USE_SELECTED_FORMAT;
+	}
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (conf_widget->prompt)))
+	{
+		gtk_widget_set_sensitive (conf_widget->list, FALSE);
+		gtk_widget_set_sensitive (conf_widget->custom_entry, FALSE);
+		gtk_widget_set_sensitive (conf_widget->custom_format_example, FALSE);
+
+		prompt_type = PROMPT_SELECTED_FORMAT;
+	}
+	else
+	{
+		g_return_if_reached ();
 	}
 
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->use_list)))
-	{
-		gtk_widget_set_sensitive (dialog->list, TRUE);
-		gtk_widget_set_sensitive (dialog->custom_entry, FALSE);
-		gtk_widget_set_sensitive (dialog->custom_format_example, FALSE);
-
-		return;
-	}
-
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->prompt)))
-	{
-		gtk_widget_set_sensitive (dialog->list, FALSE);
-		gtk_widget_set_sensitive (dialog->custom_entry, FALSE);
-		gtk_widget_set_sensitive (dialog->custom_format_example, FALSE);
-
-		return;
-	}
+	g_settings_set_enum (conf_widget->settings,
+			     PROMPT_TYPE_KEY,
+			     prompt_type);
 }
 
 static gint
@@ -708,28 +676,40 @@ get_format_from_list (GtkWidget *listview)
 
 	if (gtk_tree_selection_get_selected (selection, NULL, &iter))
 	{
-	        gint selected_value;
+		gint selected_value;
 
 		gtk_tree_model_get (model, &iter, COLUMN_INDEX, &selected_value, -1);
 
 		gedit_debug_message (DEBUG_PLUGINS, "Sel value: %d", selected_value);
 
-	        return selected_value;
+		return selected_value;
 	}
 
 	g_return_val_if_reached (0);
 }
 
-static TimeConfigureDialog *
-get_configure_dialog (GeditTimePlugin *plugin)
+static void
+on_configure_widget_selection_changed (GtkTreeSelection *selection,
+				       TimeConfigureWidget *conf_widget)
 {
-	TimeConfigureDialog *dialog = NULL;
+	gint sel_format;
+
+	sel_format = get_format_from_list (conf_widget->list);
+	g_settings_set_string (conf_widget->settings,
+			       SELECTED_FORMAT_KEY,
+			       formats[sel_format]);
+}
+
+static TimeConfigureWidget *
+get_configure_widget (GeditTimePlugin *plugin)
+{
+	TimeConfigureWidget *widget;
+	GtkTreeSelection *selection;
 	gchar *data_dir;
 	gchar *ui_file;
-	GtkWidget *content;
 	GtkWidget *viewport;
 	GeditTimePluginPromptType prompt_type;
-	gchar *sf, *cf;
+	gchar *sf;
 	GtkWidget *error_widget;
 	gboolean ret;
 	gchar *root_objects[] = {
@@ -739,42 +719,22 @@ get_configure_dialog (GeditTimePlugin *plugin)
 	
 	gedit_debug (DEBUG_PLUGINS);
 
-	dialog = g_new0 (TimeConfigureDialog, 1);
+	widget = g_slice_new (TimeConfigureWidget);
+	widget->settings = g_object_ref (plugin->priv->settings);
 
-	dialog->dialog = gtk_dialog_new_with_buttons (_("Configure insert date/time plugin..."),
-						      NULL,
-						      GTK_DIALOG_DESTROY_WITH_PARENT,
-						      GTK_STOCK_CANCEL,
-						      GTK_RESPONSE_CANCEL,
-						      GTK_STOCK_OK,
-						      GTK_RESPONSE_OK,
-						      GTK_STOCK_HELP,
-						      GTK_RESPONSE_HELP,
-						      NULL);
-
-	/* HIG defaults */
-	gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dialog->dialog)), 5);
-	gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog->dialog))),
-			     2); /* 2 * 5 + 2 = 12 */
-	gtk_container_set_border_width (GTK_CONTAINER (gtk_dialog_get_action_area (GTK_DIALOG (dialog->dialog))),
-					5);
-	gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_action_area (GTK_DIALOG (dialog->dialog))), 6);
-
-	g_return_val_if_fail (dialog->dialog != NULL, NULL);
-
-	data_dir = gedit_plugin_get_data_dir (GEDIT_PLUGIN (plugin));
+	data_dir = peas_extension_base_get_data_dir (PEAS_EXTENSION_BASE (plugin));
 	ui_file = g_build_filename (data_dir, "gedit-time-setup-dialog.ui", NULL);
 	ret = gedit_utils_get_ui_objects (ui_file,
 					  root_objects,
 					  &error_widget,
-					  "time_dialog_content", &content,
+					  "time_dialog_content", &widget->content,
 					  "formats_viewport", &viewport,
-					  "formats_tree", &dialog->list,
-					  "always_prompt", &dialog->prompt,
-					  "never_prompt", &dialog->use_list,
-					  "use_custom", &dialog->custom,
-					  "custom_entry", &dialog->custom_entry,
-					  "custom_format_example", &dialog->custom_format_example,
+					  "formats_tree", &widget->list,
+					  "always_prompt", &widget->prompt,
+					  "never_prompt", &widget->use_list,
+					  "use_custom", &widget->custom,
+					  "custom_entry", &widget->custom_entry,
+					  "custom_format_example", &widget->custom_format_example,
 					  NULL);
 
 	g_free (data_dir);
@@ -782,90 +742,82 @@ get_configure_dialog (GeditTimePlugin *plugin)
 
 	if (!ret)
 	{
-		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog->dialog))),
-		                    error_widget,
-		                    TRUE, TRUE, 0);
-		gtk_container_set_border_width (GTK_CONTAINER (error_widget), 5);
-
-		gtk_widget_show (error_widget);
-
-		return dialog;
+		return NULL;
 	}
 
-	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog->dialog), FALSE);
-
 	sf = get_selected_format (plugin);
-	create_formats_list (dialog->list, sf, plugin);
+	create_formats_list (widget->list, sf, plugin);
 	g_free (sf);
 
-	prompt_type = get_prompt_type (plugin);
+	prompt_type = g_settings_get_enum (plugin->priv->settings,
+					   PROMPT_TYPE_KEY);
 
-	cf = get_custom_format (plugin);
-     	gtk_entry_set_text (GTK_ENTRY(dialog->custom_entry), cf);
-       	g_free (cf);
+	g_settings_bind (widget->settings,
+			 CUSTOM_FORMAT_KEY,
+			 widget->custom_entry,
+			 "text",
+			 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
 
-        if (prompt_type == USE_CUSTOM_FORMAT)
-        {
-	        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->custom), TRUE);
+	if (prompt_type == USE_CUSTOM_FORMAT)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget->custom), TRUE);
 
-		gtk_widget_set_sensitive (dialog->list, FALSE);
-		gtk_widget_set_sensitive (dialog->custom_entry, TRUE);
-		gtk_widget_set_sensitive (dialog->custom_format_example, TRUE);
-        }
-        else if (prompt_type == USE_SELECTED_FORMAT)
-        {
-	        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->use_list), TRUE);
+		gtk_widget_set_sensitive (widget->list, FALSE);
+		gtk_widget_set_sensitive (widget->custom_entry, TRUE);
+		gtk_widget_set_sensitive (widget->custom_format_example, TRUE);
+	}
+	else if (prompt_type == USE_SELECTED_FORMAT)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget->use_list), TRUE);
 
-		gtk_widget_set_sensitive (dialog->list, TRUE);
-		gtk_widget_set_sensitive (dialog->custom_entry, FALSE);
-		gtk_widget_set_sensitive (dialog->custom_format_example, FALSE);
-        }
-        else
-        {
-	        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->prompt), TRUE);
+		gtk_widget_set_sensitive (widget->list, TRUE);
+		gtk_widget_set_sensitive (widget->custom_entry, FALSE);
+		gtk_widget_set_sensitive (widget->custom_format_example, FALSE);
+	}
+	else
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget->prompt), TRUE);
 
-		gtk_widget_set_sensitive (dialog->list, FALSE);
-		gtk_widget_set_sensitive (dialog->custom_entry, FALSE);
-		gtk_widget_set_sensitive (dialog->custom_format_example, FALSE);
-        }
+		gtk_widget_set_sensitive (widget->list, FALSE);
+		gtk_widget_set_sensitive (widget->custom_entry, FALSE);
+		gtk_widget_set_sensitive (widget->custom_format_example, FALSE);
+	}
 
-	updated_custom_format_example (GTK_ENTRY (dialog->custom_entry),
-			GTK_LABEL (dialog->custom_format_example));
+	updated_custom_format_example (GTK_ENTRY (widget->custom_entry),
+				       GTK_LABEL (widget->custom_format_example));
 
 	/* setup a window of a sane size. */
 	gtk_widget_set_size_request (GTK_WIDGET (viewport), 10, 200);
 
-	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog->dialog))),
-			    content, FALSE, FALSE, 0);
-	g_object_unref (content);
-	gtk_container_set_border_width (GTK_CONTAINER (content), 5);
-
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog->dialog),
-					 GTK_RESPONSE_OK);
-
-	g_signal_connect (dialog->custom,
+	g_signal_connect (widget->custom,
 			  "toggled",
-			  G_CALLBACK (configure_dialog_button_toggled),
-			  dialog);
-   	g_signal_connect (dialog->prompt,
+			  G_CALLBACK (configure_widget_button_toggled),
+			  widget);
+	g_signal_connect (widget->prompt,
 			  "toggled",
-			  G_CALLBACK (configure_dialog_button_toggled),
-			  dialog);
-	g_signal_connect (dialog->use_list,
+			  G_CALLBACK (configure_widget_button_toggled),
+			  widget);
+	g_signal_connect (widget->use_list,
 			  "toggled",
-			  G_CALLBACK (configure_dialog_button_toggled),
-			  dialog);
-	g_signal_connect (dialog->dialog,
+			  G_CALLBACK (configure_widget_button_toggled),
+			  widget);
+	g_signal_connect (widget->content,
 			  "destroy",
-			  G_CALLBACK (dialog_destroyed),
-			  dialog);
-	g_signal_connect (dialog->custom_entry,
+			  G_CALLBACK (configure_widget_destroyed),
+			  widget);
+	g_signal_connect (widget->custom_entry,
 			  "changed",
 			  G_CALLBACK (updated_custom_format_example),
-			  dialog->custom_format_example);
+			  widget->custom_format_example);
 
-	return dialog;
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget->list));
+
+	g_signal_connect (selection,
+			  "changed",
+			  G_CALLBACK (on_configure_widget_selection_changed),
+			  widget);
+
+	return widget;
 }
 
 static void
@@ -894,8 +846,12 @@ choose_format_dialog_row_activated (GtkTreeView        *list,
 	sel_format = get_format_from_list (dialog->list);
 	the_time = get_time (formats[sel_format]);
 
-	set_prompt_type (dialog->plugin, PROMPT_SELECTED_FORMAT);
-	set_selected_format (dialog->plugin, formats[sel_format]);
+	g_settings_set_enum (dialog->settings,
+			     PROMPT_TYPE_KEY,
+			     PROMPT_SELECTED_FORMAT);
+	g_settings_set_string (dialog->settings,
+			       SELECTED_FORMAT_KEY,
+			       formats[sel_format]);
 
 	g_return_if_fail (the_time != NULL);
 
@@ -920,9 +876,10 @@ get_choose_format_dialog (GtkWindow                 *parent,
 	if (parent != NULL)
 		wg = gtk_window_get_group (parent);
 
-	dialog = g_new0 (ChooseFormatDialog, 1);
+	dialog = g_slice_new (ChooseFormatDialog);
+	dialog->settings = plugin->priv->settings;
 
-	data_dir = gedit_plugin_get_data_dir (GEDIT_PLUGIN (plugin));
+	data_dir = peas_extension_base_get_data_dir (PEAS_EXTENSION_BASE (plugin));
 	ui_file = g_build_filename (data_dir, "gedit-time-dialog.ui", NULL);
 	ret = gedit_utils_get_ui_objects (ui_file,
 					  NULL,
@@ -969,7 +926,7 @@ get_choose_format_dialog (GtkWindow                 *parent,
 	}
 
 	gtk_window_group_add_window (wg,
-			     	     GTK_WINDOW (dialog->dialog));
+				     GTK_WINDOW (dialog->dialog));
 	gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), parent);
 	gtk_window_set_modal (GTK_WINDOW (dialog->dialog), TRUE);
 
@@ -978,7 +935,7 @@ get_choose_format_dialog (GtkWindow                 *parent,
 	g_free (sf);
 
 	cf = get_custom_format (plugin);
-     	gtk_entry_set_text (GTK_ENTRY(dialog->custom_entry), cf);
+	gtk_entry_set_text (GTK_ENTRY(dialog->custom_entry), cf);
 	g_free (cf);
 
 	updated_custom_format_example (GTK_ENTRY (dialog->custom_entry),
@@ -986,7 +943,7 @@ get_choose_format_dialog (GtkWindow                 *parent,
 
 	if (prompt_type == PROMPT_CUSTOM_FORMAT)
 	{
-        	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->custom), TRUE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->custom), TRUE);
 
 		gtk_widget_set_sensitive (dialog->list, FALSE);
 		gtk_widget_set_sensitive (dialog->custom_entry, TRUE);
@@ -1021,7 +978,7 @@ get_choose_format_dialog (GtkWindow                 *parent,
 			  dialog);
 	g_signal_connect (dialog->dialog,
 			  "destroy",
-			  G_CALLBACK (dialog_destroyed),
+			  G_CALLBACK (choose_format_dialog_destroyed),
 			  dialog);
 	g_signal_connect (dialog->custom_entry,
 			  "changed",
@@ -1067,8 +1024,12 @@ choose_format_dialog_response_cb (GtkWidget          *widget,
 				sel_format = get_format_from_list (dialog->list);
 				the_time = get_time (formats[sel_format]);
 
-				set_prompt_type (dialog->plugin, PROMPT_SELECTED_FORMAT);
-				set_selected_format (dialog->plugin, formats[sel_format]);
+				g_settings_set_enum (dialog->settings,
+						     PROMPT_TYPE_KEY,
+						     PROMPT_SELECTED_FORMAT);
+				g_settings_set_string (dialog->settings,
+						       SELECTED_FORMAT_KEY,
+						       formats[sel_format]);
 			}
 			else
 			{
@@ -1077,8 +1038,12 @@ choose_format_dialog_response_cb (GtkWidget          *widget,
 				format = gtk_entry_get_text (GTK_ENTRY (dialog->custom_entry));
 				the_time = get_time (format);
 
-				set_prompt_type (dialog->plugin, PROMPT_CUSTOM_FORMAT);
-				set_custom_format (dialog->plugin, format);
+				g_settings_set_enum (dialog->settings,
+						     PROMPT_TYPE_KEY,
+						     PROMPT_CUSTOM_FORMAT);
+				g_settings_set_string (dialog->settings,
+						       CUSTOM_FORMAT_KEY,
+						       format);
 			}
 
 			g_return_if_fail (the_time != NULL);
@@ -1096,43 +1061,47 @@ choose_format_dialog_response_cb (GtkWidget          *widget,
 }
 
 static void
-time_cb (GtkAction  *action,
-	 ActionData *data)
+time_cb (GtkAction       *action,
+	 GeditTimePlugin *plugin)
 {
+	GeditTimePluginPrivate *priv;
 	GtkTextBuffer *buffer;
-	gchar *the_time = NULL;
 	GeditTimePluginPromptType prompt_type;
+	gchar *the_time = NULL;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	buffer = GTK_TEXT_BUFFER (gedit_window_get_active_document (data->window));
+	priv = plugin->priv;
+
+	buffer = GTK_TEXT_BUFFER (gedit_window_get_active_document (priv->window));
 	g_return_if_fail (buffer != NULL);
 
-	prompt_type = get_prompt_type (data->plugin);
+	prompt_type = g_settings_get_enum (plugin->priv->settings,
+					   PROMPT_TYPE_KEY);
 
-        if (prompt_type == USE_CUSTOM_FORMAT)
-        {
-		gchar *cf = get_custom_format (data->plugin);
-	        the_time = get_time (cf);
+	if (prompt_type == USE_CUSTOM_FORMAT)
+	{
+		gchar *cf = get_custom_format (plugin);
+		the_time = get_time (cf);
 		g_free (cf);
 	}
-        else if (prompt_type == USE_SELECTED_FORMAT)
-        {
-		gchar *sf = get_selected_format (data->plugin);
-	        the_time = get_time (sf);
+	else if (prompt_type == USE_SELECTED_FORMAT)
+	{
+		gchar *sf = get_selected_format (plugin);
+		the_time = get_time (sf);
 		g_free (sf);
 	}
-        else
-        {
+	else
+	{
 		ChooseFormatDialog *dialog;
 
-		dialog = get_choose_format_dialog (GTK_WINDOW (data->window),
+		dialog = get_choose_format_dialog (GTK_WINDOW (priv->window),
 						   prompt_type,
-						   data->plugin);
+						   plugin);
 		if (dialog != NULL)
 		{
 			dialog->buffer = buffer;
-			dialog->plugin = data->plugin;
+			dialog->settings = plugin->priv->settings;
 
 			g_signal_connect (dialog->dialog,
 					  "response",
@@ -1152,102 +1121,60 @@ time_cb (GtkAction  *action,
 	g_free (the_time);
 }
 
-static void
-ok_button_pressed (TimeConfigureDialog *dialog)
-{
-	gint sel_format;
-	const gchar *custom_format;
-
-	gedit_debug (DEBUG_PLUGINS);
-
-	sel_format = get_format_from_list (dialog->list);
-
-	custom_format = gtk_entry_get_text (GTK_ENTRY (dialog->custom_entry));
-
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->custom)))
-	{
-		set_prompt_type (dialog->plugin, USE_CUSTOM_FORMAT);
-		set_custom_format (dialog->plugin, custom_format);
-	}
-	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->use_list)))
-	{
-		set_prompt_type (dialog->plugin, USE_SELECTED_FORMAT);
-		set_selected_format (dialog->plugin, formats [sel_format]);
-	}
-	else
-	{
-		/* Default to prompt the user with the list selected */
-		set_prompt_type (dialog->plugin, PROMPT_SELECTED_FORMAT);
-	}
-
-	gedit_debug_message (DEBUG_PLUGINS, "Sel: %d", sel_format);
-}
-
-static void
-configure_dialog_response_cb (GtkWidget           *widget,
-			      gint                 response,
-			      TimeConfigureDialog *dialog)
-{
-	switch (response)
-	{
-		case GTK_RESPONSE_HELP:
-		{
-			gedit_debug_message (DEBUG_PLUGINS, "GTK_RESPONSE_HELP");
-
-			gedit_app_show_help (gedit_app_get_default (),
-					     GTK_WINDOW (dialog),
-					     NULL,
-					     "gedit-date-time-configure");
-			break;
-		}
-		case GTK_RESPONSE_OK:
-		{
-			gedit_debug_message (DEBUG_PLUGINS, "GTK_RESPONSE_OK");
-
-			ok_button_pressed (dialog);
-
-			gtk_widget_destroy (dialog->dialog);
-			break;
-		}
-		case GTK_RESPONSE_CANCEL:
-		{
-			gedit_debug_message (DEBUG_PLUGINS, "GTK_RESPONSE_CANCEL");
-			gtk_widget_destroy (dialog->dialog);
-		}
-	}
-}
-
 static GtkWidget *
-impl_create_configure_dialog (GeditPlugin *plugin)
+gedit_time_plugin_create_configure_widget (PeasGtkConfigurable *configurable)
 {
-	TimeConfigureDialog *dialog;
+	TimeConfigureWidget *widget;
 
-	dialog = get_configure_dialog (GEDIT_TIME_PLUGIN (plugin));
+	widget = get_configure_widget (GEDIT_TIME_PLUGIN (configurable));
 
-	dialog->plugin = GEDIT_TIME_PLUGIN (plugin);
-
-	g_signal_connect (dialog->dialog,
-			  "response",
-			  G_CALLBACK (configure_dialog_response_cb),
-			  dialog);
-
-	return GTK_WIDGET (dialog->dialog);
+	return widget->content;
 }
 
 static void
 gedit_time_plugin_class_init (GeditTimePluginClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GeditPluginClass *plugin_class = GEDIT_PLUGIN_CLASS (klass);
 
 	object_class->dispose = gedit_time_plugin_dispose;
+	object_class->set_property = gedit_time_plugin_set_property;
+	object_class->get_property = gedit_time_plugin_get_property;
 
-	plugin_class->activate = impl_activate;
-	plugin_class->deactivate = impl_deactivate;
-	plugin_class->update_ui = impl_update_ui;
-
-	plugin_class->create_configure_dialog = impl_create_configure_dialog;
+	g_object_class_override_property (object_class, PROP_WINDOW, "window");
 
 	g_type_class_add_private (object_class, sizeof (GeditTimePluginPrivate));
 }
+
+static void
+gedit_time_plugin_class_finalize (GeditTimePluginClass *klass)
+{
+}
+
+static void
+peas_gtk_configurable_iface_init (PeasGtkConfigurableInterface *iface)
+{
+	iface->create_configure_widget = gedit_time_plugin_create_configure_widget;
+}
+
+static void
+gedit_window_activatable_iface_init (GeditWindowActivatableInterface *iface)
+{
+	iface->activate = gedit_time_plugin_activate;
+	iface->deactivate = gedit_time_plugin_deactivate;
+	iface->update_state = gedit_time_plugin_update_state;
+}
+
+G_MODULE_EXPORT void
+peas_register_types (PeasObjectModule *module)
+{
+	gedit_time_plugin_register_type (G_TYPE_MODULE (module));
+
+	peas_object_module_register_extension_type (module,
+						    GEDIT_TYPE_WINDOW_ACTIVATABLE,
+						    GEDIT_TYPE_TIME_PLUGIN);
+	peas_object_module_register_extension_type (module,
+						    PEAS_GTK_TYPE_CONFIGURABLE,
+						    GEDIT_TYPE_TIME_PLUGIN);
+}
+
 /* ex:ts=8:noet: */
